import mimetypes
from typing import Tuple, Optional

from urllib3 import Timeout


def upload_artifact_file(
    session,
    *,
    perftest_endpoint: str,
    attach_type: str,
    sub_type: str,
    description: str,
    filename: str,
    content_type: Optional[str] = None,
    auth_token: Optional[str] = None,
) -> Tuple[int, str]:
    if not content_type:
        guesses = mimetypes.guess_type(filename)
        if not guesses:
            raise ValueError(f"mime type could not be guessed from filename.")
        content_type = guesses[0]
        # logger.debug(f"Guessed mime-type from filename: {mime_type!r}")

    with open(filename, "rb") as f:
        content = f.read()

        return upload_artifact(
            session,
            perftest_endpoint=perftest_endpoint,
            attach_type=attach_type,
            sub_type=sub_type,
            description=description,
            content=content,
            content_type=content_type,
            auth_token=auth_token,
        )


def upload_artifact(
    session,
    *,
    perftest_endpoint: str,
    attach_type: str,
    sub_type: str,
    description: str,
    content_type: str,
    content: bytes,
    auth_token: Optional[str] = None,
) -> Tuple[int, str]:
    """

    :param session:
    :param perftest_endpoint:
    :param attach_type:
    :param sub_type:
    :param description:
    :param content_type: examples "text/csv" "text/plain" "image/svg+xml" "image/png"
    :param content:
    :return:
    """
    assert perftest_endpoint.startswith("http")
    assert "/perftest/" in perftest_endpoint
    assert not perftest_endpoint.endswith("/")
    assert not perftest_endpoint.endswith("perftest/")
    assert not perftest_endpoint.endswith("perftest")
    assert not perftest_endpoint.endswith("artifact")
    assert not perftest_endpoint.endswith("artifact/")

    headers = {
        "Content-Type": content_type,
        "X-Solidlab-Artifact-Type": attach_type,
        "X-Solidlab-Artifact-Subtype": sub_type,
        "X-Solidlab-Artifact-Description": description,
    }

    if auth_token:
        # Solidlab-Perftest-Auth is also supported
        headers["X-Solidlab-Perftest-Auth"] = auth_token

    post_attach_meta_resp = session.post(
        f"{perftest_endpoint}/artifact",
        params={},
        headers=headers,
        timeout=Timeout(connect=2.0, read=3.0),
        data=content,
    )
    post_attach_meta_resp.raise_for_status()
    artifact_url = post_attach_meta_resp.json()["@id"]
    artifact_id = post_attach_meta_resp.json()["id"]
    return artifact_id, artifact_url



import enum
from datetime import timedelta, datetime
from typing import Optional, Union, List, Dict, Any

from pydantic import BaseModel


class TestEnvStatus(enum.Enum):
    STARTING = "STARTING"
    # RUNNING says nothing about any active agent(s). Check agents for that.
    RUNNING = "RUNNING"
    STOPPING = "STOPPING"
    STOPPED = "STOPPED"
    # FAILED: Something went wrong. (So it is possible the TestEnv is partially running without agents, etc.)
    FAILED = "FAILED"


class TestEnvDef(BaseModel):
    # Describes the setup used to run tests in
    id: int
    name: str
    description: str
    # TODO especRepo: repo  # details about ESpec to use
    # fixed parameters for this environment. For example: clientCount, serverCpuCount,
    fixed_parameters: Dict[int, Union[str, int, bool]]  # parameterId to parameterValue


class TestEnv(BaseModel):
    # Describes a running test setup in which tests (= TestFragments) can be run
    id: int
    testenv_def_id: int
    slice_urn: str
    auth_token: str  # auth token needed by agent running to auth themselves to perftest-server
    status: TestEnvStatus

    # commit_id: the CSS commit currently installed. This can be changed by running tests.
    commit_id: Optional[int]
    # TODO in_use: Optional[int]  # ActiveSweepId  # locked because in use by a certain active TestFragment
